/*
 nRF24 Multi-Network TX/RX - nRF 1 (receives forward packet from nRF 2 and replies back with forward ACK)
 Version: 0.3
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 25/05/2013
 Last Modified: 29/10/2013
 
 A multi-addressable software implementation to allow for up to 255 nRF24 modules to be used at the same time.
 Each nRF24 module shares the same TX/RX address so whenever one nRF24 transmits all nearby nRF24's will receive it 
 and only accept it if it matches it's address or if it's intended for everyone. 
 The downside is that you need to allow for up to 20ms for a transmit or receive to occur.
 
 Variables include: From address (byte 0), To address (byte 1), Type (byte 2), TTL (byte 3), Data (byte 4 to 31)
 
 */
 
// ATtiny25/45/85 Pin map
//                    +-\/-+
//        Reset PB5  1|o   |8  VCC
//  nRF24L01 CE PB3  2|    |7  PB2 nRF24L01 SCK + LED + 10K resistor with button to VCC
// nRF24L01 CSN PB4  3|    |6  PB1 nRF24L01 MO
//              GND  4|    |5  PB0 nRF24L01 MI
//                    +----+

#define F_CPU 8000000 // 1 MHz

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "setup.c"
#include "mirf.c"

uint8_t data_in[32];
uint8_t data_out[32];
uint8_t neighbourNRF[10];
uint8_t neighbourNRFcount = 0;

int main(void) {
	setup();
	mirf_init();
	srand(MYADDR); // Seed the random number with our address
	_delay_ms(50);
	mirf_config();
	
	while(1) {
		RX_POWERUP;
		mirf_flush_rx_tx(); // Flush TX/RX
		mirf_CE_hi; // Start listening
		
		bool sending = false;
		
		// Wait for incoming requests
		while (!(mirf_status() & (1<<RX_DR))) {
			// Check if button is pressed
			PORTB &= ~(1<<PB2);
			DDRB &= ~(1<<PB2);
			_delay_us(10);
			
			if (PINB & (1<<PB2)) {
				DDRB |= (1<<PB2);
				sending = true;
				_delay_ms(500);
				break;
			}
			DDRB |= (1<<PB2);
			
			_delay_us(50);
		}
		mirf_CE_lo; // Stop listening
		
		if (sending == false) {
			// Read the data received and process the packet
			if (read_packet_and_process()) {
				// Blink LED
				PORTB |= (1<<PB2);
				_delay_ms(10);
				PORTB &= ~(1<<PB2);
				_delay_ms(10);
			}
		}
		else { 
			// Check for neighbouring nRF modules
			check_neighbouring_nrf();
		}
	}
}
