/*
 nRF24 Multi-Network TX/RX
 Version: 0.3
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 25/05/2013
 Last Modified: 29/10/2013
 
 */
 
 /*
	Copyright (c) 2007 Stefan Engelke <mbox@stefanengelke.de>
	
	Permission is hereby granted, free of charge, to any person 
	obtaining a copy of this software and associated documentation 
	files (the "Software"), to deal in the Software without 
	restriction, including without limitation the rights to use, copy, 
	modify, merge, publish, distribute, sublicense, and/or sell copies 
	of the Software, and to permit persons to whom the Software is 
	furnished to do so, subject to the following conditions:
	
	The above copyright notice and this permission notice shall be 
	included in all copies or substantial portions of the Software.
	
	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
	DEALINGS IN THE SOFTWARE.
	
	$Id$
*/

#ifndef _MIRF_H_
#define _MIRF_H_

#include <avr/io.h>

// Our Wifi address
#define MYADDR 3 // Change this for each module

// Mirf settings
#define mirf_CH			20
#define mirf_PAYLOAD		32
#define mirf_CONFIG		( (1<<EN_CRC) | (0<<CRCO) )
#define mirf_ADDR			(byte *)"clnt1"

// Packet array elements
#define FROM 0
#define TO 1
#define TYPE 2
#define TTL 3
#define DATA 4
#define DATAEND 31

// Packet types
#define NOACK 	0 // Tell the other nRF not to respond to us
#define YESACK 1 // Tell the other nRF to ACK our packet
#define ACKRESPOND 2 // Response by the other nRF if YESACK was received
#define FWD 3 // Tell other nRFs to forward this packet, always ACK back
#define FWDRESPOND 4 // If the packet intended for us was received by a forward, set as forward response so other nRFs forward the reply
#define NEIGHBOURREQ 5 // Request for neighbouring nRFs to reply
#define NEIGHBOUR_RESPOND 6 // Reply with this if neighbouring nRFs request received

// Return types
#define RECEIVED_NEIGHBOUR_REQ 1
#define RECEIVED_NEIGHBOUR_REPLY 2
#define RECEIVED_DATA_FROM_FORWARD 3
#define RECEIVED_DATA_FORWARD_ACK 4
#define RECEIVED_DATA_TO_FORWARD 5
#define RECEIVED_DATA_AND_ACK 6
#define RECEIVED_DATA_NO_ACK 7

// Transmit times
//#define ONE_TRANSMIT 1
//#define STD_TRANSMIT 4

// Random delay
#define NODELAY 0
#define RANDOMDELAY 1

// All Wifi address
#define ALLADDR 0

// Any from address check
#define ANYFROMADDR 1

// Pin definitions for chip select and chip enabled of the MiRF module
#define CE  PB3
#define CSN PB4

// Definitions for selecting and enabling MiRF module
#define mirf_CSN_hi	PORTB |=  (1<<CSN);
#define mirf_CSN_lo	PORTB &= ~(1<<CSN);
#define mirf_CE_hi	PORTB |=  (1<<CE);
#define mirf_CE_lo	PORTB &= ~(1<<CE);

// Public standard functions
extern void mirf_init(void);
extern void mirf_config(void);
extern void mirf_flush_rx_tx(void);
extern uint8_t mirf_status(void);
extern uint8_t mirf_data_ready(void);
extern uint8_t mirf_max_rt_reached(void);
extern uint8_t read_packet_and_process(void);
extern uint8_t mirf_listen_5ms_timeout(void);
extern void mirf_transmit_data(bool randomDelay);
extern uint8_t mirf_transmit_data_check_ack(void);
extern uint8_t check_neighbouring_nrf(void);

// Public extended functions
extern void mirf_config_register(uint8_t reg, uint8_t value);
extern void mirf_read_register(uint8_t reg, uint8_t * value, uint8_t len);
extern void mirf_write_register(uint8_t reg, uint8_t * value, uint8_t len);

#endif /* _MIRF_H_ */
